use std::path::PathBuf;

use super::Section;
use crate::error::Error;

pub trait Output<'data> {
    fn allocate(&mut self, size: u64) -> Result<u64, Error>;

    fn append_section(&mut self, section: Section<'data>) -> Result<(), Error>;

    // fn prepare_symbol<V, S>(&mut self, symbol: impl Lazy<V, S>) -> Result<&mut Self, Error>;

    fn finalize(self) -> Result<PathBuf, Error>;
}

pub struct DummyOutput;

impl<'data> Output<'data> for DummyOutput {
    fn allocate(&mut self, size: u64) -> Result<u64, Error> {
        eprintln!("Allocating: {}", size);
        Ok(size)
    }

    fn append_section(&mut self, section: Section<'data>) -> Result<(), Error> {
        eprintln!("Appending section: {}", section);
        Ok(())
    }

    fn finalize(self) -> Result<PathBuf, Error> {
        todo!();
    }
}
