use std::path::PathBuf;
use std::{fs::canonicalize, io::ErrorKind};

use crate::error::Error;

pub struct Storage {
    origin: PathBuf,
    bytes: Vec<u8>, // for storing the total if needed
}

impl Storage {
    pub fn new(origin: PathBuf) -> Self {
        Self {
            origin,
            bytes: Vec::new(),
        }
    }

    pub fn origin(&self) -> Result<PathBuf, Error> {
        canonicalize(&self.origin).map_err(|e| e.into())
    }

    pub fn destination(&self) -> Result<PathBuf, Error> {
        let mut dest = canonicalize(&self.origin)?;
        if !dest.pop() {
            let err = std::io::Error::new(ErrorKind::Other, "Destination path invalid");
            Err(Error::IOError(Box::new(err)))
        } else {
            dest.push("rld.out");
            Ok(dest)
        }
    }

    pub fn iter(&mut self) -> Result<impl Iterator<Item = &u8>, Error> {
        self.bytes = std::fs::read(&self.origin)?;

        Ok(self.bytes.iter())
    }

    pub fn bytes(&mut self) -> Result<&[u8], Error> {
        self.bytes = std::fs::read(&self.origin)?;

        Ok(&self.bytes)
    }
}
