mod common;
mod error;
mod formats;
mod linker;

use common::{Output, Storage};
use formats::{ElfObject, ElfOutput};
use linker::Linker;

fn main() {
    let mut args = std::env::args().peekable();

    // can this miss?
    args.next().unwrap();

    let mut contents: Vec<Storage> = Vec::new();
    let mut linker = Linker::elf();
    let mut maybe_out = None;

    while args.peek().is_some() {
        let fpath = args.next().expect("Unexpected peek-a-boo");
        let stored = Storage::new(fpath.into());
        contents.push(stored);
    }

    for storage in contents.iter_mut() {
        let relocatable = ElfObject::new(storage).expect("Parsing ELF");

        if maybe_out.is_none() {
            let dest = storage.destination().expect("Path resolution");
            maybe_out = Some(ElfOutput::from_object(&relocatable, dest));
        }
        linker.add_relocatable(Box::new(relocatable));
    }

    if let Some(mut output) = maybe_out {
        let linked = linker.link(&mut output).expect("Linking");
        let outpath = output.finalize().expect("Finalization");
        eprintln!("Done {:?}, linked {} bytes", outpath, linked);
    } else {
        eprintln!("Nothing to do");
    }
}
